/*
  Doc_Arduino - german Arduino Forum
  Arduino IDE 1.8.15
  Arduino Nano Every
  01.08.2021
  License: GNU GPLv3
  
  Beispiel mit der CCL – Configurable Custom Logic
  Ausgang der gesamten Logik: D5 (pinPB2)
                      ____
  (18)(PF2)(3-IN2)---| >= |
  (10)(PB1)(Event)---|  1 |---+(19)(PF3)(3-OUT)
   (9)(PB0)(Event)---|____|   |
                      LUT3    |   
                              |    ____ 
  (15)(PD2)(2-IN2)--->        +---| >= |
  (16)(PD1)(2-IN1)----------------|  1 |---+(14)(PD3)(2-OUT) 
  (17)(PD0)(2-IN0)----------------|____|   |
                                   LUT2    |
                                           |    ____
                                           +---| >= |
  (12)(PE1)(Event)-----------------------------|  1 |---+(4)(PC6)(1-OUT) 
  (11)(PE0)(Event)-----------------------------|____|   |
                                                LUT1    |
                                                        |    ____
                                                        +---| >= |
   (7)(PA1)(0-IN1)------------------------------------------|  1 |---(Event EVOUTB)(5)(PB2)
   (2)(PA0)(0-IN0)------------------------------------------|____|
                                                             LUT0
                                                                                                         
*/
/*
  Arduino Nano Every PinOut (ATmega4809)
                      _
                   __|_|__    
                  |       |  
      (D13)  PE2  +       +  PE1  (D12)  
            3.3V  +       +  PE0  (D11)  
      (AREF) PD7  +       +  PB1  (D10)(~)(TCA0 Channel 1)
  (A0)(D14)  PD3  +       +  PB0  (D 9)(~)(TCA0 Channel 0)
  (A1)(D15)  PD2  +       +  PE3  (D 8)
  (A2)(D16)  PD1  +       +  PA1  (D 7) 
  (A3)(D17)  PD0  +       +  PF4  (D 6)(~)(TCB0)
  (A4)(D18)  PF2  +       +  PB2  (D 5)(~)(TCA0 Channel 2)
  (A5)(D19)  PF3  +       +  PC6  (D 4)  
  (A6)(D20)  PD4  +       +  PF5  (D 3)(~)(TCB1)  
  (A7)(D21)  PD5  +       +  PA0  (D 2) 
            5.0V  +       +  GND
           RESET  +       +  RESET
             GND  +       +  PC5  (D 0) RX1
             VIN  +       +  PC4  (D 1) TX1      
                  |_______|  

//----------------------------------------------------------------------------------------------------------------------------------------------
!!! Arduino Nano Every with ATmega4809 !!!

 With 'default' in the table the view from the µC is meant. Not the one of the Arduino framework. 
 If you don't change anything at the portmux registers, the setting according to the board label above is valid.

 Ardu      |    Event       | EVSYS  |      CCL-LUTn     |       TCA0        |       TCBn        |  ADC0   |
  Pin Port | Port | Channel |        | default | PortMux | default | PortMux | default | PortMux | default |
   0  PC5  |  0     2 or 3  |  ----  |   ---       ---   |   ---      0-WO5  |   ---       ---   |         |
   1  PC4  |  0     2 or 3  |  ----  |   ---       ---   |   ---      0-WO4  |   ---       ---   |         |
   2  PA0  |  0     0 or 1  |  ----  |  0-IN0      ---   |  0-WO0      ---   |   ---       ---   |         |
   3  PF5  |  1     4 or 5  |  ----  |   ---       ---   |   ---      0-WO5  |   ---      1-WO   |  AIN15  |
   4  PC6  |  0     2 or 3  |  ----  |   ---      1-OUT  |   ---       ---   |   ---       ---   |         |
   5  PB2  |  1     0 or 1  | EVOUTB |   ---       ---   |   ---      0-WO2  |   ---       ---   |         |
   6  PF4  |  1     4 or 5  |  ----  |   ---       ---   |   ---      0-WO4  |   ---      0-WO   |  AIN14  |
   7  PA1  |  0     0 or 1  |  ----  |  0-IN1      ---   |  0-WO1      ---   |   ---       ---   |         |
   8  PE3  |  0     4 or 5  |  ----  |   ---       ---   |   ---      0-WO3  |   ---       ---   |  AIN11  |
   9  PB0  |  1     0 or 1  |  ----  |   ---       ---   |   ---      0-WO0  |   ---       ---   |         |
  10  PB1  |  1     0 or 1  |  ----  |   ---       ---   |   ---      0-WO1  |   ---       ---   |         |
  11  PE0  |  0     4 or 5  |  ----  |   ---       ---   |   ---      0-WO0  |   ---       ---   |  AIN8   |
  12  PE1  |  0     4 or 5  |  ----  |   ---       ---   |   ---      0-WO1  |   ---       ---   |  AIN9   |
  13  PE2  |  0     4 or 5  | EVOUTE |   ---       ---   |   ---      0-WO2  |   ---       ---   |  AIN10  |
  14  PD3  |  1     2 or 3  |  ----  |  2-OUT      ---   |   ---      0-WO3  |   ---       ---   |  AIN3   |
  15  PD2  |  1     2 or 3  | EVOUTD |  2-IN2      ---   |   ---      0-WO2  |   ---       ---   |  AIN2   |
  16  PD1  |  1     2 or 3  |  ----  |  2-IN1      ---   |   ---      0-WO1  |   ---       ---   |  AIN1   |
  17  PD0  |  1     2 or 3  |  ----  |  2-IN0      ---   |   ---      0-WO0  |   ---       ---   |  AIN0   |
  18  PF2  |  1     4 or 5  | EVOUTF |  3-IN2      ---   |   ---      0-WO2  |   ---       ---   |  AIN12  |  board internal double assignment with PA2 observe !!! (EVOUTA)
  19  PF3  |  1     4 or 5  |  ----  |  3-OUT      ---   |   ---      0-WO3  |   ---       ---   |  AIN13  |  board internal double assignment with PA3 observe !!!
  20  PD4  |  1     2 or 3  |  ----  |   ---       ---   |   ---      0-WO4  |   ---       ---   |  AIN4   |
  21  PD5  |  1     2 or 3  |  ----  |   ---       ---   |   ---      0-WO5  |   ---       ---   |  AIN5   |

  Wahrheitstabelle:
  IN2 | IN1 | IN0 | LUTnOUT [bit number in CCL.TRUTHn]
   0  |  0  |  0  | TRUTHn[0]
   0  |  0  |  1  | TRUTHn[1]
   0  |  1  |  0  | TRUTHn[2]
   0  |  1  |  1  | TRUTHn[3]
   1  |  0  |  0  | TRUTHn[4]
   1  |  0  |  1  | TRUTHn[5]
   1  |  1  |  0  | TRUTHn[6]
   1  |  1  |  1  | TRUTHn[7]                   
*/

#include <NanoEveryPin.h>
#include <NanoEveryPinNamen.h>

// LUT-3
InputPin  <pinPF2> LUT3in2; 
InputPin  <pinPB1> LUT3in1; // D10, Ersatzpin mittels Eventsystem
InputPin  <pinPB0> LUT3in0; // D9,  Ersatzpin mittels Eventsystem

// LUT-2
InputPin  <pinPD1> LUT2in1;
InputPin  <pinPD0> LUT2in0; 

// LUT-1
InputPin  <pinPE1> LUT1in1; // D12, Ersatzpin mittels Eventsystem
InputPin  <pinPE0> LUT1in0; // D11, Ersatzpin mittels Eventsystem

// LUT-0
InputPin  <pinPA1> LUT0in1; 
InputPin  <pinPA0> LUT0in0; 
OutputPin <pinPB2> LUT0out; // D5, Ersatzpin mittels Eventsystem

void setup (void)
{
  // LUT-3
  LUT3in2.init(); LUT3in2.enablePullup(); LUT3in2.enableInvertIO();
  LUT3in1.init(); LUT3in1.enablePullup(); LUT3in1.enableInvertIO();
  LUT3in0.init(); LUT3in0.enablePullup(); LUT3in0.enableInvertIO();

  // LUT-2
  LUT2in1.init(); LUT2in1.enablePullup(); LUT2in1.enableInvertIO();
  LUT2in0.init(); LUT2in0.enablePullup(); LUT2in0.enableInvertIO();
  
  // LUT-1
  LUT1in1.init(); LUT1in1.enablePullup(); LUT1in1.enableInvertIO();
  LUT1in0.init(); LUT1in0.enablePullup(); LUT1in0.enableInvertIO();

  // LUT-0
  LUT0in1.init(); LUT0in1.enablePullup(); LUT0in1.enableInvertIO();
  LUT0in0.init(); LUT0in0.enablePullup(); LUT0in0.enableInvertIO();
  LUT0out.init(); 

  initEventSystem();
  initLUT3CCL();
  initLUT2CCL();
  initLUT1CCL();
  initLUT0CCL();
  CCL.CTRLA = CCL_ENABLE_bm;           // general enable CCL module
}

void loop (void)
{ }

void initEventSystem(void)
{
  // für LUT3 ---------
  EVSYS.CHANNEL1 = EVSYS_GENERATOR_PORT1_PIN1_gc;   // verbinde Kanal 1 mit Generator 'Pin D10' (PB1)
  EVSYS.USERCCLLUT3B = EVSYS_CHANNEL_CHANNEL1_gc;   // verbinde User 'CCL-LUT3B' mit Kanal 1

  EVSYS.CHANNEL0 = EVSYS_GENERATOR_PORT1_PIN0_gc;   // verbinde Kanal 0 mit Generator 'Pin D9' (PB0)
  EVSYS.USERCCLLUT3A = EVSYS_CHANNEL_CHANNEL0_gc;   // verbinde User 'CCL-LUT3A' mit Kanal 0

  // für LUT1 ---------
  EVSYS.CHANNEL5 = EVSYS_GENERATOR_PORT0_PIN1_gc;   // verbinde Kanal 5 mit Generator 'Pin D12' (PE1)
  EVSYS.USERCCLLUT1B = EVSYS_CHANNEL_CHANNEL5_gc;   // verbinde User 'CCL-LUT1B' mit Kanal 5

  EVSYS.CHANNEL4 = EVSYS_GENERATOR_PORT0_PIN0_gc;   // verbinde Kanal 4 mit Generator 'Pin D11' (PE0)
  EVSYS.USERCCLLUT1A = EVSYS_CHANNEL_CHANNEL4_gc;   // verbinde User 'CCL-LUT1A' mit Kanal 4

  // für LUT0 ---------
  EVSYS.CHANNEL3 = EVSYS_GENERATOR_CCL_LUT0_gc;     // verbinde Kanal 3 mit Generator 'CCL LUT0-OUT' (PB2)
  EVSYS.USEREVOUTB = EVSYS_CHANNEL_CHANNEL3_gc;     // verbinde (PB2) User 'USEREVOUTB' mit Kanal 3
}

void initLUT3CCL(void)
{
   CCL.LUT3CTRLC  = CCL_INSEL2_IO_gc;     // LUT3-IN2 input source from Pin
   CCL.LUT3CTRLB  = CCL_INSEL1_EVENTB_gc; // LUT3-IN1 input source from Event
   CCL.LUT3CTRLB |= CCL_INSEL0_EVENTA_gc; // LUT3-IN0 input source from Event
   
   CCL.TRUTH3 = 0b1111'1110;              // Configure Truth Table, OR  
   
   /* OUTEN_bm wird nur benötigt, wenn der Logik-Ausgang LUTn-OUT µC intern auf seinen Pin geschalten werden soll
   CCL.LUT3CTRLA  = CCL_OUTEN_bm;         // enable Output LUT3     */
   CCL.LUT3CTRLA = CCL_ENABLE_bm;         // enable LUT3
}

void initLUT2CCL(void)
{
   CCL.LUT2CTRLC  = CCL_INSEL2_LINK_gc;   // LUT2-IN2 input source from LUT3-OUT
   CCL.LUT2CTRLB  = CCL_INSEL1_IO_gc;     // LUT2-IN1 input source from Pin
   CCL.LUT2CTRLB |= CCL_INSEL0_IO_gc;     // LUT2-IN0 input source from Pin
  
   CCL.TRUTH2 = 0b1111'1110;              // Configure Truth Table, OR

   /* OUTEN_bm wird nur benötigt, wenn der Logik-Ausgang LUTn-OUT µC intern auf seinen Pin geschalten werden soll
   CCL.LUT2CTRLA  = CCL_OUTEN_bm;         // enable Output LUT2     */
   CCL.LUT2CTRLA |= CCL_ENABLE_bm;        // enable LUT2
}

void initLUT1CCL(void)
{
  CCL.LUT1CTRLC  = CCL_INSEL2_LINK_gc;   // LUT1-IN2 input source from LUT2-OUT
  CCL.LUT1CTRLB  = CCL_INSEL1_EVENTB_gc; // LUT1-IN1 input source from Event
  CCL.LUT1CTRLB |= CCL_INSEL0_EVENTA_gc; // LUT1-IN0 input source from Event
   
  CCL.TRUTH1 = 0b1111'1110;              // Configure Truth Table, OR

  /* OUTEN_bm wird nur benötigt, wenn der LUTn Ausgang µC intern auf seinen Pin geschalten werden soll
  CCL.LUT1CTRLA  = CCL_OUTEN_bm;         // enable Output LUT1     */
  CCL.LUT1CTRLA |= CCL_ENABLE_bm;        // enable LUT1
}

void initLUT0CCL(void)
{
  CCL.LUT0CTRLC  = CCL_INSEL2_LINK_gc;   // LUT0-IN2 input source from LUT1-OUT
  CCL.LUT0CTRLB  = CCL_INSEL1_IO_gc;     // LUT0-IN1 input source from Pin
  CCL.LUT0CTRLB |= CCL_INSEL0_IO_gc;     // LUT0-IN0 input source from Pin
   
  CCL.TRUTH0 = 0b1111'1110;              // Configure Truth Table, OR

  /* OUTEN_bm wird nur benötigt, wenn der LUTn Ausgang µC intern auf seinen Pin geschalten werden soll
  CCL.LUT0CTRLA  = CCL_OUTEN_bm;         // enable Output LUT0     */
  CCL.LUT0CTRLA |= CCL_ENABLE_bm;        // enable LUT0 Unit
}
